#include <iostream>
#include "bitmanip.h"

// ----------------------------------------------------------------
unsigned bit_reverse(
	unsigned in,
	unsigned num_bits)
{
	unsigned out;

	out = in;

	out = ((out & 0xaaaaaaaa) >>  1) | ((out & 0x55555555) <<  1);
	out = ((out & 0xcccccccc) >>  2) | ((out & 0x33333333) <<  2);
	out = ((out & 0xf0f0f0f0) >>  4) | ((out & 0x0f0f0f0f) <<  4);
	out = ((out & 0xff00ff00) >>  8) | ((out & 0x00ff00ff) <<  8);
	out = ((out & 0xffff0000) >> 16) | ((out & 0x0000ffff) << 16);

	out >>= (32 - num_bits);

	return out;
}

// ----------------------------------------------------------------
unsigned splog2(
	unsigned n)
{
	int rv = 0;
	int nsv = n;

	if (n == 0) {
		std::cerr << "splog2():  Can't take logarithm of zero.\n";
		exit(1);
	}

	while (n != 1) {
		n >>= 1;
		rv++;
	}

	if (nsv != (1 << rv)) {
		std::cerr << "splog2():  Argument "
			<< nsv << " is not a power of two.\n";
		exit(1);
	}

	return rv;
}

// ----------------------------------------------------------------
unsigned is_a_power_of_two(
	unsigned n)
{
	if (n == 0)
		return 0;
	if ((n & (n - 1)) == 0)
		return 1;
	return 0;
}

unsigned is_a_power_of_four(
	unsigned n)
{
	if (!is_a_power_of_two(n))
		return 0;
	unsigned l = splog2(n);
	l >>= 1;
	l <<= 1;
	if (n == (1U << l))
		return 1;
	return 0;
}
