#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include "bitmanip.h"
#include "twiddles.h"
#include "dft_dpfp.h"

// ----------------------------------------------------------------
// y_k = \sum_{j=1}^N w_{kj} x_j

void dft_dpfp(
	complex_double * y,
	fft_opts_t       opts,
	fft_verbosity_t  vopts)
{
	int N = opts.N;
	int k, j;
	complex_double * temp = new complex_double[N];
	for (k = 0; k < N; k++)
		temp[k] = y[k];

	if (opts.fold_out) {
		for (k = 1; k < N; k+=2)
			temp[k] = -temp[k];
	}

	tvector<complex_double> W = full_twiddle_table_double(N, opts.forward);
	double recip_sqrt_N = sqrt((double)1.0 / (double)N);

	for (k = 0; k < N; k++) {
		y[k] = 0.0;
		for (j = 0; j < N; j++) {
			y[k] += W[ (k * j) % N ] * temp[j];
		}
	}

	if (opts.scale) {
		for (k = 0; k < N; k++)
			y[k] *= recip_sqrt_N;
	}

	if (opts.fold_in) {
		for (k = 1; k < N; k+=2)
			y[k] = -y[k];
	}
	delete [] temp;
}
