#include <math.h>
#include "twiddles.h"

// ----------------------------------------------------------------
complex_float make_W_float(int k, int N)
{
	return make_W_float(k, N, 0);
}

complex_float make_W_float(int k, int N, int forward)
{
	float arg = 2.0 * M_PI * (float)k / (float)N;
	if (forward)
		arg = -arg;
	return complex_float(cos(arg), sin(arg));
}

tvector<complex_float> half_twiddle_table_float(int N, int forward)
{
	tvector<complex_float> table(N/2);
	for (int k = 0; k < N/2; k++)
		table[k] = make_W_float(k, N, forward);
	return table;
}

tvector<complex_float> full_twiddle_table_float(int N, int forward)
{
	tvector<complex_float> table(N);
	for (int k = 0; k < N; k++)
		table[k] = make_W_float(k, N, forward);
	return table;
}

// ----------------------------------------------------------------
complex_double make_W_double(int k, int N)
{
	return make_W_double(k, N, 0);
}

complex_double make_W_double(int k, int N, int forward)
{
	double arg = 2.0 * M_PI * (double)k / (double)N;
	if (forward)
		arg = -arg;
	return complex_double(cos(arg), sin(arg));
}

tvector<complex_double> half_twiddle_table_double(int N, int forward)
{
	tvector<complex_double> table(N/2);
	for (int k = 0; k < N/2; k++)
		table[k] = make_W_double(k, N, forward);
	return table;
}

tvector<complex_double> full_twiddle_table_double(int N, int forward)
{
	tvector<complex_double> table(N);
	for (int k = 0; k < N; k++)
		table[k] = make_W_double(k, N, forward);
	return table;
}
