#include<stdio.h>
#include "complex_float.h"
#include "tmatrix.h"

// ----------------------------------------------------------------
static void usage(
	char * argv0)
{
	fprintf(stderr, "Usage: %s {-r|-c} {num rows} {num cols} {decimation #} {input file} {output file}\n", argv0);
	fprintf(stderr, "Decimation number must divide num rows and num cols.\n");
	exit(1);
}

// ----------------------------------------------------------------
static void usage(
	char * argv0);

// ----------------------------------------------------------------
int main(int argc, char ** argv)
{
	int do_complex = 0;
	int anr, anc, m;
	int bnr, bnc;
	int ai, aj, bi, bj;
	char * input_file_name;
	char * output_file_name;

	if (argc != 7)
		usage(argv[0]);
	if (strcmp(argv[1], "-r") == 0)
		do_complex = 0;
	else if (strcmp(argv[1], "-c") == 0)
		do_complex = 1;
	else
		usage(argv[0]);
	if (sscanf(argv[2], "%d", &anr) != 1)
		usage(argv[0]);
	if (sscanf(argv[3], "%d", &anc) != 1)
		usage(argv[0]);
	if (sscanf(argv[4], "%d", &m) != 1)
		usage(argv[0]);
	if (((anr % m) != 0) || ((anc % m) != 0))
		usage(argv[0]);
	input_file_name  = argv[5];
	output_file_name = argv[6];

	bnr = anr / m;
	bnc = anc / m;

	if (do_complex) {
		tmatrix<complex_float> A(anr, anc);
		tmatrix<complex_float> B(bnr, bnc);
		A.load_from_binary_file_or_die(input_file_name);
		for (ai = 0, bi = 0; ai < anr; ai += m, bi++)
			for (aj = 0, bj = 0; aj < anc; aj += m, bj++)
				B[bi][bj] = A[ai][aj];
		B.store_to_binary_file_or_die(output_file_name);
	}
	else {
		tmatrix<float> A(anr, anc);
		tmatrix<float> B(bnr, bnc);
		A.load_from_binary_file_or_die(input_file_name);
		for (ai = 0, bi = 0; ai < anr; ai += m, bi++)
			for (aj = 0, bj = 0; aj < anc; aj += m, bj++)
				B[bi][bj] = A[ai][aj];
		B.store_to_binary_file_or_die(output_file_name);
	}

	return 0;
}
