#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "tvector.h"
#include "complex_float.h"

// ----------------------------------------------------------------
static void usage(char * argv0)
{
	fprintf(stderr, "Usage: %s {N} {k ...} {output_file_name}\n",
		argv0);
	exit(1);
}

// ----------------------------------------------------------------
int main(
	int     argc,
	char ** argv)
{
	int argi, N, i, j;
	char * output_file_name;
	if (argc < 4)
		usage(argv[0]);
	if (sscanf(argv[1], "%d", &N) != 1)
		usage(argv[0]);
	output_file_name = argv[argc-1];
	int numk = argc - 3;

	tvector<float> kfreq(numk);
	tvector<float> kmag(numk);
	tvector<float> kphz(numk);
	tvector<complex_float>csdata(N);
	for (argi = 2, i = 0; argi < (argc - 1); argi++, i++) {
		float freq = 0.0, mag = 1.0, phz = 0.0;
		if (sscanf(argv[argi], "%f:%f:%f", &freq, &mag, &phz) == 3) {
			;
		}
		else if (sscanf(argv[argi], "%f:%f", &freq, &mag) == 2) {
			phz = 0.0;
		}
		else if (sscanf(argv[argi], "%f", &freq) == 1) {
			mag = 1.0;
			phz = 0.0;
		}
		phz *= 2.0 * M_PI;

		kfreq[i] = freq;
		kmag[i]  = mag;
		kphz[i]  = phz;
	}

	for (j = 0; j < N; j++) {
		csdata[j] = 0.0;
		for (i = 0; i < numk; i++) {
			float arg = 2.0 * M_PI * kfreq[i] * (float)j / (float)N;
			csdata[j].re += kmag[i] * cos(arg + kphz[i]);
			csdata[j].im += kmag[i] * sin(arg + kphz[i]);
		}
	}

	csdata.store_to_binary_file(output_file_name);

	return 0;
}
