#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "tvector.h"
#include "complex_float.h"
#include "complex_double.h"
#include "comparef.h"
#include "compared.h"
#include "comparecf.h"
#include "comparecd.h"

// ----------------------------------------------------------------
static void usage(char * argv0)
{
	fprintf(stderr, "Usage: %s {-rs|-rd|-cs|-cd} {input file 1} {input file 2}\n",
		argv0);
	exit(1);
}

// ----------------------------------------------------------------
int main(
	int     argc,
	char ** argv)
{
	int    argb = 1;
	char * input_file_name_1;
	char * input_file_name_2;
	int    do_complex = 0;
	int    do_double  = 0;
	int    show_file_names = 0;
	float  tol = 1e-6;
	int    rc  = 1;

	if (argc < 2)
		usage(argv[0]);
	while ((argb < argc) && (argv[argb][0] == '-')) {
		if (strcmp(argv[argb], "-rs") == 0) {
			do_complex = 0;
			do_double  = 0;
		}
		else if (strcmp(argv[argb], "-rd") == 0) {
			do_complex = 0;
			do_double  = 1;
		}
		else if (strcmp(argv[argb], "-cs") == 0) {
			do_complex = 1;
			do_double  = 0;
		}
		else if (strcmp(argv[argb], "-cd") == 0) {
			do_complex = 1;
			do_double  = 1;
		}
		else if (strcmp(argv[argb], "-sfn") == 0) {
			show_file_names = 1;
		}
		else if (strcmp(argv[argb], "-tol") == 0) {
			if ((argc - argb) < 2)
				usage(argv[0]);
			argb++;
			if (sscanf(argv[argb], "%f", &tol) != 1)
				usage(argv[0]);
		}
		else {
			usage(argv[0]);
		}
		argb++;
	}
	if ((argc - argb) != 2)
		usage(argv[0]);
	input_file_name_1 = argv[argb];
	input_file_name_2 = argv[argb + 1];

	if (show_file_names)
		printf("%s vs. %s:\n", input_file_name_1, input_file_name_2);

	if (do_complex) {
		if (do_double) {
			tvector<complex_double> in1, in2, diff;
			int n;
			double rosserr;
			if (!in1.load_from_binary_file(input_file_name_1)) {
				fprintf(stderr, "%s:  Couldn't read \"%s\".\n",
					argv[0], input_file_name_1);
				exit(1);
			}
			if (!in2.load_from_binary_file(input_file_name_2)) {
				fprintf(stderr, "%s:  Couldn't read \"%s\".\n",
					argv[0], input_file_name_2);
				exit(1);
			}
			diff = in1 - in2;
			n = in1.get_num_elements();
			// <a-b,a-b> / <a,a>
			double numer = 0.0;
			double denom = 0.0;
			for (int i = 0; i < n; i++) {
				numer += diff[i].pwr();
				denom += in1[i].pwr();
			}
			rosserr = sqrt(numer/denom);
			printf("Ross metric %.6le\n", rosserr);
			rc = (rosserr <= tol) ? 0 : 1;
		}

		else {
			tvector<complex_float> in1, in2, diff;
			int n;
			float rosserr;
			if (!in1.load_from_binary_file(input_file_name_1)) {
				fprintf(stderr, "%s:  Couldn't read \"%s\".\n",
					argv[0], input_file_name_1);
				exit(1);
			}
			if (!in2.load_from_binary_file(input_file_name_2)) {
				fprintf(stderr, "%s:  Couldn't read \"%s\".\n",
					argv[0], input_file_name_2);
				exit(1);
			}
			diff = in1 - in2;
			n = in1.get_num_elements();
			// <a-b,a-b> / <a,a>
			float numer = 0.0;
			float denom = 0.0;
			for (int i = 0; i < n; i++) {
				numer += diff[i].pwr();
				denom += in1[i].pwr();
			}
			rosserr = sqrt(numer/denom);
			printf("Ross metric %.6e\n", rosserr);
			rc = (rosserr <= tol) ? 0 : 1;
		}
	}

	else {
		if (do_double) {
			tvector<double> in1, in2, diff;
			int n;
			double rosserr;
			if (!in1.load_from_binary_file(input_file_name_1)) {
				fprintf(stderr, "%s:  Couldn't read \"%s\".\n",
					argv[0], input_file_name_1);
				exit(1);
			}
			if (!in2.load_from_binary_file(input_file_name_2)) {
				fprintf(stderr, "%s:  Couldn't read \"%s\".\n",
					argv[0], input_file_name_2);
				exit(1);
			}
			diff = in1 - in2;
			n = in1.get_num_elements();
			// <a-b,a-b> / <a,a>
			double numer = 0.0;
			double denom = 0.0;
			for (int i = 0; i < n; i++) {
				numer += diff[i] * diff[i];
				denom += in1[i] * in1[i];
			}
			rosserr = sqrt(numer/denom);
			printf("Ross metric %.6le\n", rosserr);
			rc = (rosserr <= tol) ? 0 : 1;
		}

		else {
			tvector<float> in1, in2, diff;
			int n;
			float rosserr;
			if (!in1.load_from_binary_file(input_file_name_1)) {
				fprintf(stderr, "%s:  Couldn't read \"%s\".\n",
					argv[0], input_file_name_1);
				exit(1);
			}
			if (!in2.load_from_binary_file(input_file_name_2)) {
				fprintf(stderr, "%s:  Couldn't read \"%s\".\n",
					argv[0], input_file_name_2);
				exit(1);
			}
			diff = in1 - in2;
			n = in1.get_num_elements();

			// <a-b,a-b> / <a,a>
			float numer = 0.0;
			float denom = 0.0;
			for (int i = 0; i < n; i++) {
				numer += diff[i] * diff[i];
				denom += in1[i] * in1[i];
			}
			rosserr = sqrt(numer/denom);
			printf("Ross metric %.6e\n", rosserr);
			rc = (rosserr <= tol) ? 0 : 1;
		}
	}

	if (show_file_names)
		printf("\n");

	return rc;
}
