#include "tvector.h"
#include "complex_float.h"
#include "complex_double.h"

// ----------------------------------------------------------------
static void usage(char * argv0)
{
	fprintf(stderr, "Usage: %s {num bins} {lo} {hi} {input file}.\n", argv0);
	exit(1);
}

// ----------------------------------------------------------------
int main(int argc, char ** argv)
{
	int argb = 1;
	char * input_file_name = 0;
	int num_bins;
	float lo, hi;
	int * bins = 0;
	tvector<float> v;
	int i, n;

	if (argc < 2)
		usage(argv[0]);
	while (argv[argb][0] == '-') {
		if (strcmp(argv[argb], "--help") == 0)
			usage(argv[0]);
		else
			usage(argv[0]);
		argb++;
	}

	if ((argc - argb) != 4)
		usage(argv[0]);
	if (sscanf(argv[argb], "%d", &num_bins) != 1)
		usage(argv[0]);
	if (sscanf(argv[argb+1], "%f", &lo) != 1)
		usage(argv[0]);
	if (sscanf(argv[argb+2], "%f", &hi) != 1)
		usage(argv[0]);
	input_file_name = argv[argb+3];

	if (lo >= hi) {
		fprintf(stderr, "lo must be < hi.\n");
		usage(argv[0]);
	}
	if (num_bins < 1) {
		fprintf(stderr, "num_bins must be >= 1.\n");
		usage(argv[0]);
	}

	if (!v.load_from_binary_file(input_file_name)) {
		fprintf(stderr, "%s:  Couldn't read \"%s\".\n",
			argv[0], input_file_name);;
		exit(1);
	}
	n = v.get_num_elements();

	bins = new int[num_bins];
	for (i = 0; i < num_bins; i++)
		bins[i] = 0;

	for (i = 0; i < n; i++) {
		int idx = 0;
		float input = v[i];

		if (input < lo)
			continue;
		if (input >= hi)
			continue;

		idx = (int)((input-lo) / (hi - lo) * num_bins);
		bins[idx]++;
	}

	for (i = 0; i < num_bins; i++) {
		float x = lo + (hi - lo) * (float)i / (float)num_bins;
		printf("%11.7g\t%d\n", x, bins[i]);
	}


	delete [] bins;
	return 0;
}
