#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "fixed_point.h"

// ----------------------------------------------------------------
static void usage(char * argv0)
{
	fprintf(stderr, "Usage: %s {nb} {nf} {fixeds ...}\n", argv0);
	exit(1);
}

// ----------------------------------------------------------------
int main(int argc, char ** argv)
{
	float fp;
	fixed_point fx;
	int nb, nf;

	if (argc < 4)
		usage(argv[0]);
	if (sscanf(argv[1], "%d", &nb) != 1)
		usage(argv[0]);
	if (sscanf(argv[2], "%d", &nf) != 1)
		usage(argv[0]);

	for (int argi = 3; argi < argc; argi++) {
		if (strcmp(argv[argi], "range") == 0) {
			fixed_point fxlo(1LL << (nb-1), nb);
			fixed_point fxhi((1LL << (nb-1)) - 1, nb);
			float fplo = fxlo.to_float(nf);
			float fphi = fxhi.to_float(nf);
			printf("Range   %0*llx to %0*llx  %11.7f to %11.7f\n",
				(nb+3)>>2, fxlo.get_value(), (nb+3)>>2, fxhi.get_value(),
				fplo, fphi);
			continue;
		}

		s64 value;
		if (sscanf(argv[argi], "%llx", &value) != 1)
			usage(argv[0]);
		fx = fixed_point(value, nb);
		fp = fx.to_float(nf);
		if (fabs(fp) < 1.0e-5)
			printf("%0*llx -> %11.7le\n", (nb+3)>>2, fx.get_value(), fp);
		else
			printf("%0*llx -> %11.7lf\n", (nb+3)>>2, fx.get_value(), fp);
	}

	return 0;
}
