#include "tvector.h"
#include "complex_float.h"
#include "complex_double.h"

// ----------------------------------------------------------------
static void usage(char * argv0)
{
	fprintf(stderr, "Usage: %s [-rs|-rs|-cs|-cd] {input file}.\n", argv0);
	exit(1);
}

// ----------------------------------------------------------------
int main(int argc, char ** argv)
{
	int do_double = 0;
	int do_complex = 0;
	int argb = 1;
	char * input_file_name = 0;

	if (argc < 2)
		usage(argv[0]);
	while ((argb < argc) && (argv[argb][0] == '-')) {
		if (strcmp(argv[argb], "--help") == 0)
			usage(argv[0]);

		else if (strcmp(argv[argb], "-rs") == 0) {
			do_complex = 0;
			do_double  = 0;
		}
		else if (strcmp(argv[argb], "-rd") == 0) {
			do_complex = 0;
			do_double  = 1;
		}
		else if (strcmp(argv[argb], "-cs") == 0) {
			do_complex = 1;
			do_double  = 0;
		}
		else if (strcmp(argv[argb], "-cd") == 0) {
			do_complex = 1;
			do_double  = 1;
		}

		else
			usage(argv[0]);
		argb++;
	}

	if ((argc - argb) < 1)
		usage(argv[0]);

	for (int argi = argb; argi < argc; argi++) {
		input_file_name = argv[argi];

		if (do_double) {

			if (do_complex) {
				tvector<complex_double> x;
				int n;
				double f, sum;

				if (!x.load_from_binary_file(input_file_name)) {
					fprintf(stderr, "%s:  Couldn't read \"%s\".\n",
						argv[0], input_file_name);;
					exit(1);
				}

				n = x.get_num_elements();
				sum = 0.0;
				for (int i = 0; i < n; i++) {
					f = x[i].mag();
					sum += f * f;
				}
				sum /= n;
				printf("%s: %18.11lg\n", input_file_name, sum);
			}
			else {
				tvector<double> x;
				int n;
				double sum;

				if (!x.load_from_binary_file(input_file_name)) {
					fprintf(stderr, "%s:  Couldn't read \"%s\".\n",
						argv[0], input_file_name);;
					exit(1);
				}

				n = x.get_num_elements();
				sum = 0.0;
				for (int i = 0; i < n; i++)
					sum += x[i] * x[i];
				sum /= n;
				printf("%s: %18.11lg\n", input_file_name, sum);
			}

		}
		else {

			if (do_complex) {
				tvector<complex_float> x;
				int n;
				float f, sum;

				if (!x.load_from_binary_file(input_file_name)) {
					fprintf(stderr, "%s:  Couldn't read \"%s\".\n",
						argv[0], input_file_name);;
					exit(1);
				}

				n = x.get_num_elements();
				sum = 0.0;
				for (int i = 0; i < n; i++) {
					f = x[i].mag();
					sum += f * f;
				}
				sum /= n;
				printf("%s: %11.7g\n", input_file_name, sum);
			}
			else {
				tvector<float> x;
				int n;
				float sum;

				if (!x.load_from_binary_file(input_file_name)) {
					fprintf(stderr, "%s:  Couldn't read \"%s\".\n",
						argv[0], input_file_name);;
					exit(1);
				}

				n = x.get_num_elements();
				sum = 0.0;
				for (int i = 0; i < n; i++)
					sum += x[i] * x[i];
				sum /= n;
				printf("%s: %11.7g\n", input_file_name, sum);
			}

		}

	}

	return 0;
}
