#include "tvector.h"

// ----------------------------------------------------------------
static void usage(char * argv0)
{
	fprintf(stderr, "Usage: %s [-sp|-dp] [-std] {input file}.\n", argv0);
	fprintf(stderr, "With -std, also compute the standard deviation.\n");
	exit(1);
}

// ----------------------------------------------------------------
int main(int argc, char ** argv)
{
	int do_double = 0;
	int do_stddev = 0;
	int argb = 1;
	char * input_file_name = 0;

	if (argc < 2)
		usage(argv[0]);
	while ((argb < argc) && (argv[argb][0] == '-')) {
		if (strcmp(argv[argb], "--help") == 0)
			usage(argv[0]);
		else if (strcmp(argv[argb], "-sp") == 0)
			do_double  = 0;
		else if (strcmp(argv[argb], "-dp") == 0)
			do_double  = 1;
		else if (strcmp(argv[argb], "-std") == 0)
			do_stddev  = 1;
		else
			usage(argv[0]);
		argb++;
	}

	if ((argc - argb) < 1)
		usage(argv[0]);

	for (int argi = argb; argi < argc; argi++) {
		input_file_name = argv[argi];

		if (do_double) {
			tvector<double> x;
			int n;
			double mean;
			if (!x.load_from_binary_file(input_file_name)) {
				fprintf(stderr, "%s:  Couldn't read \"%s\".\n",
					argv[0], input_file_name);;
				exit(1);
			}

			n = x.get_num_elements();
			mean = 0.0;
			for (int i = 0; i < n; i++)
				mean += x[i];
			mean /= n;
			printf("%s: %18.11lg", input_file_name, mean);

			if (do_stddev) {
				double stddev = 0.0;
				double curdiff;
				for (int i = 0; i < n; i++) {
					curdiff = x[i] - mean;
					stddev += curdiff * curdiff;
				}
				stddev /= n - 1; // Warning: blows up at n = 1!
				stddev = sqrt(stddev);
				printf("  %11.7lg", stddev);
			}
			printf("\n");
		}
		else {
			tvector<float> x;
			int n;
			float mean;
			if (!x.load_from_binary_file(input_file_name)) {
				fprintf(stderr, "%s:  Couldn't read \"%s\".\n",
					argv[0], input_file_name);;
				exit(1);
			}

			n = x.get_num_elements();
			mean = 0.0;
			for (int i = 0; i < n; i++)
				mean += x[i];
			mean /= n;
			printf("%s: %11.7g", input_file_name, mean);

			if (do_stddev) {
				float stddev = 0.0;
				float curdiff;
				for (int i = 0; i < n; i++) {
					curdiff = x[i] - mean;
					stddev += curdiff * curdiff;
				}
				stddev /= n - 1; // Warning: blows up at n = 1!
				stddev = sqrt(stddev);
				printf("  %11.7g", stddev);
			}
			printf("\n");
		}
	}

	return 0;
}
