#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "complex_float.h"
#include "complex_double.h"
#include "tmatrix.h"

typedef struct _opts_t {
	int    do_complex;
	int    do_double;
	int    do_mul;
	int    num_rows;
	int    num_cols;
	double scalar;
	char * input_file_name;
	char * output_file_name;
} opts_t;

static void set_defaults(opts_t * popts);
static int parse_command_line(int argc, char ** argv, opts_t * popts);

// ----------------------------------------------------------------
static void usage(char * argv0)
{
	fprintf(stderr, "Usage: %s {-r|-c} {-sp|-dp} {scalar} {# rows} {# cols} {input file} "
		"{output file}\n", argv0);
	exit(1);
}

// ----------------------------------------------------------------
int main(
	int     argc,
	char ** argv)
{
	opts_t opts;

	set_defaults(&opts);
	if (!parse_command_line(argc, argv, &opts))
		usage(argv[0]);

	if (opts.do_double) {
		if (opts.do_complex) {
			tmatrix<complex_double> A(opts.num_rows, opts.num_cols);
			A.load_from_binary_file(opts.input_file_name);
			complex_double s(opts.scalar);
			if (opts.do_mul)
				A *= s;
			else {
				int nr = A.get_num_rows();
				int nc = A.get_num_cols();
				for (int i = 0; i < nr; i++)
					for (int j = 0; j < nc; j++)
						A[i][j] += s;
			}
			A.store_to_binary_file(opts.output_file_name);
		}
		else {
			tmatrix<double> A(opts.num_rows, opts.num_cols);
			A.load_from_binary_file(opts.input_file_name);
			if (opts.do_mul)
				A *= opts.scalar;
			else {
				int nr = A.get_num_rows();
				int nc = A.get_num_cols();
				for (int i = 0; i < nr; i++)
					for (int j = 0; j < nc; j++)
						A[i][j] += opts.scalar;
			}
			A.store_to_binary_file(opts.output_file_name);
		}
	}
	else {
		if (opts.do_complex) {
			tmatrix<complex_float> A(opts.num_rows, opts.num_cols);
			A.load_from_binary_file(opts.input_file_name);
			complex_float s((float)opts.scalar);
			if (opts.do_mul)
				A *= s;
			else {
				int nr = A.get_num_rows();
				int nc = A.get_num_cols();
				for (int i = 0; i < nr; i++)
					for (int j = 0; j < nc; j++)
						A[i][j] += s;
			}
			A.store_to_binary_file(opts.output_file_name);
		}
		else {
			tmatrix<float> A(opts.num_rows, opts.num_cols);
			A.load_from_binary_file(opts.input_file_name);
			if (opts.do_mul)
				A *= (float)opts.scalar;
			else {
				int nr = A.get_num_rows();
				int nc = A.get_num_cols();
				for (int i = 0; i < nr; i++)
					for (int j = 0; j < nc; j++)
						A[i][j] += opts.scalar;
			}
			A.store_to_binary_file(opts.output_file_name);
		}
	}

	return 0;
}

// ----------------------------------------------------------------
static void set_defaults(opts_t * popts)
{
	popts->do_complex       = 1;
	popts->do_double        = 0;
	popts->do_mul           = 1;
	popts->scalar           = 1.0;
	popts->num_rows         = 1;
	popts->num_cols         = 1;
	popts->input_file_name  = 0;
	popts->output_file_name = 0;
}

// ----------------------------------------------------------------
static int parse_command_line(int argc, char ** argv, opts_t * popts)
{
	int argi;
	for (argi = 1; argi < argc; argi++) {
		if (strcmp(argv[argi], "--help") == 0)
			return 0;
		if (strcmp(argv[argi], "--") == 0)
			break;
		if (argv[argi][0] != '-')
			break;

		else if (strcmp(argv[argi], "-r") == 0)
			popts->do_complex = 0;
		else if (strcmp(argv[argi], "-c") == 0)
			popts->do_complex = 1;

		else if (strcmp(argv[argi], "-m") == 0)
			popts->do_mul = 1;
		else if (strcmp(argv[argi], "-a") == 0)
			popts->do_mul = 0;

		else if (strcmp(argv[argi], "-sp") == 0)
			popts->do_double = 0;
		else if (strcmp(argv[argi], "-dp") == 0)
			popts->do_double = 1;

		else {
			return 0;
		}
	}
	if ((argc - argi) != 5)
		return 0;

	if (sscanf(argv[argi], "%lf", &popts->scalar) != 1)
		return 0;
	if (sscanf(argv[argi+1], "%d", &popts->num_rows) != 1)
		return 0;
	if (sscanf(argv[argi+2], "%d", &popts->num_cols) != 1)
		return 0;
	popts->input_file_name  = argv[argi+3];
	popts->output_file_name = argv[argi+4];

	return 1;
}
