#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "tvector.h"
#include "complex_float.h"
#include "complex_double.h"
#include "comparef.h"
#include "compared.h"
#include "comparecf.h"
#include "comparecd.h"

// ----------------------------------------------------------------
static void usage(char * argv0)
{
	fprintf(stderr, "Usage: %s {-rs|-rd|-cs|-cd} {input file}\n",
		argv0);
	exit(1);
}

// ----------------------------------------------------------------
int main(
	int     argc,
	char ** argv)
{
	char * input_file_name;
	int    do_complex;
	int    do_double;
	float  tol = 1e-6;

	if (argc != 3)
		usage(argv[0]);
	if (strcmp(argv[1], "-rs") == 0) {
		do_complex = 0;
		do_double  = 0;
	}
	else if (strcmp(argv[1], "-rd") == 0) {
		do_complex = 0;
		do_double  = 1;
	}
	else if (strcmp(argv[1], "-cs") == 0) {
		do_complex = 1;
		do_double  = 0;
	}
	else if (strcmp(argv[1], "-cd") == 0) {
		do_complex = 1;
		do_double  = 1;
	}
	else {
		usage(argv[0]);
	}
	input_file_name = argv[2];

	if (do_complex) {
		if (do_double) {
			tvector<complex_double> in, zero, diff;
			double  maxrelerr, rmsrelerr, maxabserr, maxabsval;
			if (!in.load_from_binary_file(input_file_name)) {
				fprintf(stderr, "%s:  Couldn't read \"%s\".\n",
					argv[0], input_file_name);
				exit(1);
			}
			zero = tvector<complex_double>(complex_double(0.0, 0.0),
				in.get_num_elements());
			(void)complex_double_vector_compare(in.expose(), zero.expose(),
				in.get_num_elements(), tol, maxrelerr, rmsrelerr,
				maxabserr, maxabsval);
			printf("Max mag %.6le\n", maxrelerr);
			printf("RMS mag %.6le\n", rmsrelerr);
			printf("Max abs %.6le\n", maxabserr);
		}
		else {
			tvector<complex_float> in, zero, diff;
			float  maxrelerr, rmsrelerr, maxabserr, maxabsval;
			if (!in.load_from_binary_file(input_file_name)) {
				fprintf(stderr, "%s:  Couldn't read \"%s\".\n",
					argv[0], input_file_name);
				exit(1);
			}
			zero = tvector<complex_float>(complex_float(0.0, 0.0),
				in.get_num_elements());
			(void)complex_float_vector_compare(in.expose(), zero.expose(),
				in.get_num_elements(), tol, maxrelerr, rmsrelerr,
				maxabserr, maxabsval);
			printf("Max mag %.6e\n", maxrelerr);
			printf("RMS mag %.6e\n", rmsrelerr);
			printf("Max abs %.6e\n", maxabserr);
		}
	}
	else {
		if (do_double) {
			tvector<double> in, zero, diff;
			double  maxrelerr, rmsrelerr, maxabserr, maxabsval;
			if (!in.load_from_binary_file(input_file_name)) {
				fprintf(stderr, "%s:  Couldn't read \"%s\".\n",
					argv[0], input_file_name);
				exit(1);
			}
			zero = tvector<double>(0.0, in.get_num_elements());
			(void)double_vector_compare(in.expose(), zero.expose(),
				in.get_num_elements(), tol, maxrelerr, rmsrelerr,
				maxabserr, maxabsval);
			printf("Max mag %.6le\n", maxrelerr);
			printf("RMS mag %.6le\n", rmsrelerr);
			printf("Max abs %.6le\n", maxabserr);
		}
		else {
			tvector<float> in, zero, diff;
			float  maxrelerr, rmsrelerr, maxabserr, maxabsval;
			if (!in.load_from_binary_file(input_file_name)) {
				fprintf(stderr, "%s:  Couldn't read \"%s\".\n",
					argv[0], input_file_name);
				exit(1);
			}
			zero = tvector<float>(0.0, in.get_num_elements());
			(void)float_vector_compare(in.expose(), zero.expose(),
				in.get_num_elements(), tol, maxrelerr, rmsrelerr,
				maxabserr, maxabsval);
			printf("Max mag %.6e\n", maxrelerr);
			printf("RMS mag %.6e\n", rmsrelerr);
			printf("Max abs %.6e\n", maxabserr);
		}
	}

	return 0;
}
