#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "tvector.h"
#include "complex_float.h"

// ----------------------------------------------------------------
static void usage(char * argv0)
{
	fprintf(stderr, "Usage: %s {N} {k ...} {output_file_name}\n",
		argv0);
	exit(1);
}

// ----------------------------------------------------------------
int main(
	int     argc,
	char ** argv)
{
	int argi, N, i, j;
	char * output_file_name;
	if (argc < 4)
		usage(argv[0]);
	if (sscanf(argv[1], "%d", &N) != 1)
		usage(argv[0]);
	output_file_name = argv[argc-1];
	int numk = argc - 3;

	tvector<int>   loc(numk);
	tvector<float> mag(numk);
	tvector<float> phz(numk);
	tvector<complex_float>csdata(N);
	for (argi = 2, i = 0; argi < (argc - 1); argi++, i++) {
		loc[i] = 0;
		mag[i] = 1.0;
		phz[i] = 0.0;
		if (sscanf(argv[argi], "%d:%f:%f", &loc[i], &mag[i], &phz[i]) == 3) {
			;
		}
		else if (sscanf(argv[argi], "%d:%f", &loc[i], &mag[i]) == 2) {
			phz[i] = 0.0;
		}
		else if (sscanf(argv[argi], "%d", &loc[i]) == 1) {
			mag[i] = 1.0;
			phz[i] = 0.0;
		}
		phz[i] *= 2.0 * M_PI;
	}

	for (j = 0; j < N; j++) {
		csdata[j] = 0.0;
		for (i = 0; i < numk; i++) {
			if (j == loc[i]) {
				csdata[j].re += mag[i] * cos(phz[i]);
				csdata[j].im += mag[i] * sin(phz[i]);
			}
		}
	}

	csdata.store_to_binary_file(output_file_name);

	return 0;
}
