#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "complex_float.h"
#include "tmatrix.h"

typedef struct _opts_t {
	int    do_complex;
	int    num_rows;
	int    num_cols;
	char * output_file_name;
	int    xyamp_argb;
} opts_t;

static void set_defaults(opts_t * popts);
static int parse_command_line(int argc, char ** argv, opts_t * popts);

// ----------------------------------------------------------------
static void usage(char * argv0)
{
	fprintf(stderr, "Usage: %s {-r|-c} {# rows} {# cols} {output file} "
		"{x:y:amp ...}\n", argv0);
	exit(1);
}

// ----------------------------------------------------------------
int main(
	int     argc,
	char ** argv)
{
	opts_t opts;
	int i, j;

	set_defaults(&opts);
	if (!parse_command_line(argc, argv, &opts))
		usage(argv[0]);

	tmatrix<complex_float> cfdata(opts.num_rows, opts.num_cols);
	for (i = 0; i < opts.num_rows; i++)
		for (j = 0; j < opts.num_cols; j++)
			cfdata[i][j] = 0.0;

	for (int argi = opts.xyamp_argb; argi < argc; argi++) {
		float mag, phz;
		complex_float c = 0.0;
		if (sscanf(argv[argi], "%d:%d:%f,%f", &i, &j, &mag, &phz) == 4) {
		}
		else if (sscanf(argv[argi], "%d:%d:%f", &i, &j, &mag) == 3) {
			phz = 0.0;
		}
		else if (sscanf(argv[argi], "%d:%d", &i, &j) == 2) {
			mag = 1.0;
			phz = 0.0;
		}
		c.re = mag * cos(phz);
		c.im = mag * sin(phz);
		if ((i >= 0) && (i < opts.num_rows)) {
			if ((j >= 0) && (j < opts.num_cols)) {
				cfdata[i][j] += c;
			}
		}
	}

	if (opts.do_complex) {
		cfdata.store_to_binary_file(opts.output_file_name);
	}
	else {
		tmatrix<float> fdata(opts.num_rows, opts.num_cols);
		for (i = 0; i < opts.num_rows; i++)
			for (j = 0; j < opts.num_cols; j++)
				fdata[i][j] = cfdata[i][j].re;
		fdata.store_to_binary_file(opts.output_file_name);
	}

	return 0;
}

// ----------------------------------------------------------------
static void set_defaults(opts_t * popts)
{
	popts->do_complex       = 1;
	popts->num_rows         = 1;
	popts->num_cols         = 1;
	popts->output_file_name = 0;
	popts->xyamp_argb       = 0;
}

// ----------------------------------------------------------------
static int parse_command_line(int argc, char ** argv, opts_t * popts)
{
	int argi;
	for (argi = 1; argi < argc; argi++) {
		if (strcmp(argv[argi], "--help") == 0)
			return 0;
		if (argv[argi][0] != '-')
			break;

		else if (strcmp(argv[argi], "-r") == 0) {
			popts->do_complex = 0;
		}
		else if (strcmp(argv[argi], "-c") == 0) {
			popts->do_complex = 1;
		}

		else {
			return 0;
		}
	}
	if ((argc - argi) < 4)
		return 0;
	if (sscanf(argv[argi], "%d", &popts->num_rows) != 1)
		return 0;
	if (sscanf(argv[argi+1], "%d", &popts->num_cols) != 1)
		return 0;
	popts->output_file_name = argv[argi+2];
	popts->xyamp_argb = argi+3;

	return 1;
}
