#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "tvector.h"
#include "complex_float.h"
#include "complex_double.h"
#include "comparef.h"
#include "compared.h"
#include "comparecf.h"
#include "comparecd.h"

void find_range_float(
	tvector<float> & v,
	float          & fmin,
	float          & fmax);

void find_range_double(
	tvector<double> & v,
	double          & fmin,
	double          & fmax);

void find_range_complex_float(
	tvector<complex_float> & v,
	float                  & fmin,
	float                  & fmax);

void find_range_complex_double(
	tvector<complex_double> & v,
	double                  & fmin,
	double                  & fmax);

// ----------------------------------------------------------------
static void usage(char * argv0)
{
	fprintf(stderr, "Usage: %s {-rs|-rd|-cs|-cd} {input file}\n",
		argv0);
	exit(1);
}

// ----------------------------------------------------------------
int main(
	int     argc,
	char ** argv)
{
	char * input_file_name;
	int    do_complex = 0;
	int    do_double = 0;
	int    argb = 1;

	if (argc < 2)
		usage(argv[0]);
	while ((argb < argc) && (argv[argb][0] == '-')) {
		if (strcmp(argv[argb], "--help") == 0)
			usage(argv[0]);

		else if (strcmp(argv[argb], "-rs") == 0) {
			do_complex = 0;
			do_double  = 0;
		}
		else if (strcmp(argv[argb], "-rd") == 0) {
			do_complex = 0;
			do_double  = 1;
		}
		else if (strcmp(argv[argb], "-cs") == 0) {
			do_complex = 1;
			do_double  = 0;
		}
		else if (strcmp(argv[argb], "-cd") == 0) {
			do_complex = 1;
			do_double  = 1;
		}

		else
			usage(argv[0]);
		argb++;
	}

	if ((argc - argb) < 1)
		usage(argv[0]);

	for (int argi = argb; argi < argc; argi++) {
		input_file_name = argv[argi];

		if (do_complex) {
			if (do_double) {
				tvector<complex_double> v;
				double  fmin, fmax;
				if (!v.load_from_binary_file(input_file_name)) {
					fprintf(stderr, "%s:  Couldn't read \"%s\".\n",
						argv[0], input_file_name);
					exit(1);
				}
				find_range_complex_double(v, fmin, fmax);
				printf("%s: min %11.7lg max %11.7lg\n",
					input_file_name, fmin, fmax);
			}
			else {
				tvector<complex_float> v;
				float fmin, fmax;
				if (!v.load_from_binary_file(input_file_name)) {
					fprintf(stderr, "%s:  Couldn't read \"%s\".\n",
						argv[0], input_file_name);
					exit(1);
				}
				find_range_complex_float(v, fmin, fmax);
				printf("%s: min %11.7g max %11.7g\n",
					input_file_name, fmin, fmax);
			}
		}

		else {
			if (do_double) {
				tvector<double> v;
				double fmin, fmax;
				if (!v.load_from_binary_file(input_file_name)) {
					fprintf(stderr, "%s:  Couldn't read \"%s\".\n",
						argv[0], input_file_name);
					exit(1);
				}
				find_range_double(v, fmin, fmax);
				printf("%s: min %11.7lg max %11.7lg\n",
					input_file_name, fmin, fmax);
			}
			else {
				tvector<float> v;
				float fmin, fmax;
				if (!v.load_from_binary_file(input_file_name)) {
					fprintf(stderr, "%s:  Couldn't read \"%s\".\n",
						argv[0], input_file_name);
					exit(1);
				}
				find_range_float(v, fmin, fmax);
				printf("%s: min %11.7g max %11.7g\n",
					input_file_name, fmin, fmax);
			}
		}
	}

	return 0;
}

// ----------------------------------------------------------------
void find_range_float(
	tvector<float> & v,
	float          & fmin,
	float          & fmax)
{
	int n = v.get_num_elements();

	fmin = v[0];
	fmax = v[0];
	for (int i = 0; i < n; i++) {
		float fcur = v[i];
		if (fcur < fmin)
			fmin = fcur;
		if (fcur > fmax)
			fmax = fcur;
	}
}

// ----------------------------------------------------------------
void find_range_double(
	tvector<double> & v,
	double          & fmin,
	double          & fmax)
{
	int n = v.get_num_elements();

	fmin = v[0];
	fmax = v[0];

	for (int i = 0; i < n; i++) {
		double fcur = v[i];
		if (fcur < fmin)
			fmin = fcur;
		if (fcur > fmax)
			fmax = fcur;
	}
}

// ----------------------------------------------------------------
void find_range_complex_float(
	tvector<complex_float> & v,
	float                  & fmin,
	float                  & fmax)
{
	int n = v.get_num_elements();

	fmin = v[0].mag();
	fmax = fmin;
	for (int i = 0; i < n; i++) {
		float cur = v[i].mag();
		if (cur < fmin)
			fmin = cur;
		if (cur > fmax)
			fmax = cur;
	}
}

// ----------------------------------------------------------------
void find_range_complex_double(
	tvector<complex_double> & v,
	double                  & fmin,
	double                  & fmax)
{
	int n = v.get_num_elements();

	fmin = v[0].mag();
	fmax = fmin;
	for (int i = 0; i < n; i++) {
		double cur = v[i].mag();
		if (cur < fmin)
			fmin = cur;
		if (cur > fmax)
			fmax = cur;
	}
}
