#ifndef COMPLEX_DOUBLE_H
#define COMPLEX_DOUBLE_H

#include <iostream>
#include <sstream>
#include <iomanip>

class complex_double {
public:

	complex_double(void)
	{
		this->re = 0.0;
		this->im = 0.0;
	}

	complex_double(double r)
	{
		this->re = r;
		this->im = 0.0;
	}

	complex_double(double r, double i)
	{
		this->re = r;
		this->im = i;
	}

	complex_double(const complex_double & that)
	{
		this->re = that.re;
		this->im = that.im;
	}

	complex_double & operator=(complex_double that);
	complex_double   operator+(complex_double that) const;
	complex_double   operator-(complex_double that) const;
	complex_double   operator*(complex_double that) const;
	complex_double   operator/(complex_double that);

	complex_double   operator-(void) const;

	complex_double & operator=(double r);
	complex_double   operator+(double r) const;
	complex_double   operator-(double r) const;
	complex_double   operator*(double r);
	complex_double   operator/(double r);

	complex_double & operator+=(complex_double that);
	complex_double & operator-=(complex_double that);
	complex_double & operator*=(complex_double that);
	complex_double & operator/=(complex_double & that);

	complex_double & operator+=(double r);
	complex_double & operator-=(double r);
	complex_double & operator*=(double r);
	complex_double & operator/=(double r);

	int operator==(complex_double that) const;
	int operator!=(complex_double that) const;

	int operator==(double r) const;
	int operator!=(double r) const;

	double pwr(void);
	double mag(void);
	double phz(void);
	complex_double conj(void);

	friend std::ostream & operator<<(std::ostream & os,
		const complex_double & c);
	friend std::istream & operator>>(std::istream & is,
		complex_double & c);
	int from_string(char * string);
	void pretty_print(std::ostream & os) const;

public:
	double re;
	double im;
};

#endif // COMPLEX_DOUBLE_H
