#include <math.h>
#include "complex_fixed_point.h"

// ----------------------------------------------------------------
complex_fixed_point::complex_fixed_point(void)
{
	// Use default constructors for real and imaginary parts.
}

complex_fixed_point::complex_fixed_point(fixed_point r, fixed_point i)
{
	this->re = r;
	this->im = i;
}

complex_fixed_point::complex_fixed_point(const complex_fixed_point & that)
{
	this->re = that.re;
	this->im = that.im;
}

// ----------------------------------------------------------------
complex_fixed_point::complex_fixed_point(complex_float c,
	int num_bits, int num_frac)
{
	this->re = fixed_point(c.re, num_bits, num_frac);
	this->im = fixed_point(c.im, num_bits, num_frac);
}

complex_float complex_fixed_point::to_complex_float(int num_frac)
{
	complex_float rv;
	rv.re = this->re.to_float(num_frac);
	rv.im = this->im.to_float(num_frac);
	return rv;
}

// ----------------------------------------------------------------
complex_fixed_point & complex_fixed_point::operator=(complex_fixed_point that)
{
	this->re = that.re;
	this->im = that.im;
	return *this;
}

complex_fixed_point complex_fixed_point::operator+(
	complex_fixed_point that)
{
	complex_fixed_point rv;
	rv.re = this->re + that.re;
	rv.im = this->im + that.im;
	return rv;
}

complex_fixed_point complex_fixed_point::operator-(
	complex_fixed_point that)
{
	complex_fixed_point rv;
	rv.re = this->re - that.re;
	rv.im = this->im - that.im;
	return rv;
}

complex_fixed_point complex_fixed_point::operator*(
	complex_fixed_point that)
{
	complex_fixed_point rv;

	rv.re = this->re * that.re - this->im * that.im;
	rv.im = this->re * that.im + this->im * that.re;
	return rv;
}

complex_fixed_point complex_fixed_point::operator-(void)
{
	complex_fixed_point rv;
	rv.re = -this->re;
	rv.im = -this->im;
	return rv;
}

// ----------------------------------------------------------------
complex_fixed_point & complex_fixed_point::operator+=(complex_fixed_point that)
{
	*this = *this + that;
	return *this;
}

complex_fixed_point & complex_fixed_point::operator-=(complex_fixed_point that)
{
	*this = *this - that;
	return *this;
}

complex_fixed_point & complex_fixed_point::operator*=(complex_fixed_point that)
{
	*this = *this * that;
	return *this;
}

// ----------------------------------------------------------------
int complex_fixed_point::operator==(complex_fixed_point that)
{
	if (this->re != that.re)
		return 0;
	if (this->im != that.im)
		return 0;
	return 1;
}

int complex_fixed_point::operator!=(complex_fixed_point that)
{
	return !(*this == that);
}

// ----------------------------------------------------------------
std::ostream & operator<<(std::ostream & os, const complex_fixed_point & cplx)
{
	os << cplx.re << "," << cplx.im;
	return os;
}

std::istream & operator>>(std::istream & is, complex_fixed_point & cplx)
{
	int c;

	// Skip over whitespace.
	c = is.peek();
	if (c == EOF) {
		is.setstate(std::ios::failbit);
		return is;
	}
	while ((c == ' ') || (c == '\t') || (c == '\n')) {
		is.ignore(1, c);
		c = is.peek();
	}
	if (c == EOF) {
		is.setstate(std::ios::failbit);
		return is;
	}

	// Read real part
	is >> cplx.re;
	if (is.fail()) {
		is.setstate(std::ios::failbit);
		return is;
	}

	// Read imaginary part, if any.
	c = is.peek();
	if (is.fail()) {
		is.clear();
		return is;
	}
	else if (c == EOF) {
		is.setstate(std::ios::eofbit);
		return is;
	}
	else if (c != ',') {
		return is;
	}
	else {
		(void)is.get();
		is >> cplx.im;
		if (is.fail()) {
			is.setstate(std::ios::failbit);
			return is;
		}
	}

	return is;
}
