#include <math.h>
#include "complex_float.h"

// ----------------------------------------------------------------
complex_float & complex_float::operator=(complex_float that)
{
	this->re = that.re;
	this->im = that.im;
	return *this;
}

complex_float complex_float::operator+(complex_float that) const
{
	complex_float rv;
	rv.re = this->re + that.re;
	rv.im = this->im + that.im;
	return rv;
}

complex_float complex_float::operator-(complex_float that) const
{
	complex_float rv;
	rv.re = this->re - that.re;
	rv.im = this->im - that.im;
	return rv;
}

complex_float complex_float::operator*(complex_float that) const
{
	complex_float rv;

	rv.re = this->re * that.re - this->im * that.im;
	rv.im = this->re * that.im + this->im * that.re;
	return rv;
}

complex_float complex_float::operator/(complex_float that)
{
	complex_float numer = *this * that.conj();
	float denom = that.pwr();
	complex_float rv(numer.re / denom, numer.im / denom);
	return rv;
}

complex_float complex_float::operator-(void) const
{
	complex_float rv;
	rv.re = - this->re;
	rv.im = - this->im;
	return rv;
}

// ----------------------------------------------------------------
complex_float & complex_float::operator=(float r)
{
	this->re = r;
	this->im = 0.0;
	return *this;
}

complex_float complex_float::operator+(float r) const
{
	complex_float rv;
	rv.re = this->re + r;
	rv.im = this->im;
	return rv;
}

complex_float complex_float::operator-(float r) const
{
	complex_float rv;
	rv.re = this->re - r;
	rv.im = this->im;
	return rv;
}

complex_float complex_float::operator*(float r)
{
	complex_float rv;
	rv.re = this->re * r;
	rv.im = this->im * r;
	return rv;
}

complex_float complex_float::operator/(float r)
{
	complex_float rv;
	rv.re = this->re / r;
	rv.im = this->im / r;
	return rv;
}

// ----------------------------------------------------------------
complex_float & complex_float::operator+=(complex_float that)
{
	*this = *this + that;
	return *this;
}

complex_float & complex_float::operator-=(complex_float that)
{
	*this = *this - that;
	return *this;
}

complex_float & complex_float::operator*=(complex_float that)
{
	*this = *this * that;
	return *this;
}

complex_float & complex_float::operator/=(complex_float & that)
{
	*this = *this / that;
	return *this;
}

// ----------------------------------------------------------------
complex_float & complex_float::operator+=(float r)
{
	*this = *this + r;
	return *this;
}

complex_float & complex_float::operator-=(float r)
{
	*this = *this - r;
	return *this;
}

complex_float & complex_float::operator*=(float r)
{
	*this = *this * r;
	return *this;
}

complex_float & complex_float::operator/=(float r)
{
	*this = *this / r;
	return *this;
}

// ----------------------------------------------------------------
int complex_float::operator==(complex_float that) const
{
	if (this->re != that.re)
		return 0;
	if (this->im != that.im)
		return 0;
	return 1;
}

int complex_float::operator!=(complex_float that) const
{
	return !(*this == that);
}

int complex_float::operator==(float r) const
{
	if (this->re != r)
		return 0;
	if (this->im != 0.0)
		return 0;
	return 1;
}

int complex_float::operator!=(float r) const
{
	return !(*this == r);
}

// ----------------------------------------------------------------
float complex_float::pwr(void)
{
	return this->re * this->re + this->im * this->im;
}

float complex_float::mag(void)
{
	return sqrt(this->pwr());
}

float complex_float::phz(void)
{
	return atan2(this->im, this->re);
}

complex_float complex_float::conj(void)
{
	complex_float rv;
	rv.re =  this->re;
	rv.im = -this->im;
	return rv;
}

// ----------------------------------------------------------------
std::ostream & operator<<(std::ostream & os, const complex_float & cplx)
{
	os << cplx.re << "," << cplx.im;
	return os;
}

std::istream & operator>>(std::istream & is, complex_float & cplx)
{
	int c;

	// Skip over whitespace.
	c = is.peek();
	if (c == EOF) {
		is.setstate(std::ios::failbit);
		return is;
	}
	while ((c == ' ') || (c == '\t') || (c == '\n')) {
		is.ignore(1, c);
		c = is.peek();
	}
	if (c == EOF) {
		is.setstate(std::ios::failbit);
		return is;
	}

	// Read real part
	is >> cplx.re;
	if (is.fail()) {
		is.setstate(std::ios::failbit);
		return is;
	}

	// Read imaginary part, if any.
	cplx.im = 0.0;
	c = is.peek();
	if (is.fail()) {
		is.clear();
		return is;
	}
	else if (c == EOF) {
		is.setstate(std::ios::eofbit);
		return is;
	}
	else if (c != ',') {
		return is;
	}
	else {
		(void)is.get();
		is >> cplx.im;
		if (is.fail()) {
			is.setstate(std::ios::failbit);
			return is;
		}
	}

	return is;
}

// ----------------------------------------------------------------
void complex_float::pretty_print(std::ostream & os) const
{
	float tol = 1e-5;
	complex_float rounded, out(*this);
	rounded.re = floor(this->re + 0.5);
	rounded.im = floor(this->im + 0.5);
	if (fabs(rounded.re - this->re) < tol)
		out.re = rounded.re;
	if (fabs(rounded.im - this->im) < tol)
		out.im = rounded.im;
	if ((out.re == 0.0) && (out.im == 0.0)) {
		os << "0";
	}
	else if (out.re == 0.0) {
		if (out.im == -1.0)
			os<< "-";
		else if (out.im != 1.0)
			os<< out.im;
		os << "i";
	}
	else if (out.im == 0.0) {
		os << out.re;
	}
	else {
		os << out.re;
		if (out.im == 1.0)
			os << "+";
		else if (out.im == -1.0)
			os << "-";
		else if (out.im < 0.0)
			os << "-" << -out.im;
		else
			os << "+" << out.im;
		os << "i";
	}
}

// ----------------------------------------------------------------
int complex_float::from_string(char * string)
{
	std::istringstream iss(string, std::ios_base::in);
	iss >> *this;
	return iss.fail() ? 0 : 1;
}
