#ifndef COMPLEX_FLOAT_H
#define COMPLEX_FLOAT_H

#include <iostream>
#include <sstream>
#include <iomanip>

class complex_float {
public:

	complex_float(void)
	{
		this->re = 0.0;
		this->im = 0.0;
	}

	complex_float(float r)
	{
		this->re = r;
		this->im = 0.0;
	}

	complex_float(float r, float i)
	{
		this->re = r;
		this->im = i;
	}

	complex_float(const complex_float & that)
	{
		this->re = that.re;
		this->im = that.im;
	}

	complex_float & operator=(complex_float that);
	complex_float   operator+(complex_float that) const;
	complex_float   operator-(complex_float that) const;
	complex_float   operator*(complex_float that) const;
	complex_float   operator/(complex_float that);

	complex_float   operator-(void) const;

	complex_float & operator=(float r);
	complex_float   operator+(float r) const;
	complex_float   operator-(float r) const;
	complex_float   operator*(float r);
	complex_float   operator/(float r);

	complex_float & operator+=(complex_float that);
	complex_float & operator-=(complex_float that);
	complex_float & operator*=(complex_float that);
	complex_float & operator/=(complex_float & that);

	complex_float & operator+=(float r);
	complex_float & operator-=(float r);
	complex_float & operator*=(float r);
	complex_float & operator/=(float r);

	int operator==(complex_float that) const;
	int operator!=(complex_float that) const;

	int operator==(float r) const;
	int operator!=(float r) const;

	float pwr(void);
	float mag(void);
	float phz(void);
	complex_float conj(void);

	friend std::ostream & operator<<(std::ostream & os,
		const complex_float & cplx);
	friend std::istream & operator>>(std::istream & is,
		complex_float & cplx);
	void pretty_print(std::ostream & os) const;
	int from_string(char * string);

public:
	float re;
	float im;
};

#endif // COMPLEX_FLOAT_H
