#ifndef DDA_H
#define DDA_H

#include <iostream>
#include <sstream>
#include <iomanip>
#include "fixed_point.h"

class dda_t {
public:
	dda_t(void);

	dda_t(fixed_point c0);
	dda_t(fixed_point c0, fixed_point c1);
	dda_t(fixed_point c0, fixed_point c1, fixed_point c2);
	dda_t(fixed_point * pseeds, int nseeds);

	dda_t(float c0, int num_bits, int num_frac);
	dda_t(float c0, float c1, int num_bits, int num_frac);
	dda_t(float c0, float c1, float c2, int num_bits, int num_frac);
	dda_t(float * pseeds, int nseeds, int num_bits, int num_frac);

	dda_t(const dda_t & that);
	dda_t & operator=(dda_t that);
	~dda_t(void);

	void set_reg(int regno, fixed_point new_value);
	fixed_point get_reg(int regno);
	fixed_point & operator[](int regno);
	int get_num_regs(void);

	fixed_point get(void);
	void step(void);
	fixed_point get_and_step(void);

	friend std::ostream & operator<<(std::ostream & os,
		const dda_t & dda);

private:
	int nregs;
	fixed_point * pregs;

	void bounds_check(int regno);
};

#endif // DDA_H
