// From _Numerical Recipes in C_.
// Returns pseudorandom real or complex floats with Gaussian distribution.
// Complexes have Gaussian-distributed magnitude, and phase uniformly
// distributed between 0 and 2 pi.

// Routines ending in "_r" are re-entrant.

// psdes.h is used as the random-integer source; look there for seeding
// information.

#ifndef GASDEV_H
#define GASDEV_H

#include "psdes.h"
#include "complex_float.h"
#include "complex_double.h"

// ----------------------------------------------------------------
float float_gasdev(void);
float float_gasdev_ms(float mean, float stddev);
complex_float complex_float_gasdev(void);
complex_float complex_float_gasdev_ms( float mean, float stddev);

// ----------------------------------------------------------------
float float_gasdev_r(unsigned * pstate0, unsigned * pstate1);
float float_gasdev_ms_r(float mean, float stddev,
	unsigned * pstate0, unsigned * pstate1);
complex_float complex_float_gasdev_r(unsigned * pstate0, unsigned * pstate1);
complex_float complex_float_gasdev_ms_r(float mean, float stddev,
	unsigned * pstate0, unsigned * pstate1);

// ----------------------------------------------------------------
double double_gasdev(void);
double double_gasdev_ms(double mean, double stddev);
complex_double complex_double_gasdev(void);
complex_double complex_double_gasdev_ms( double mean, double stddev);

// ----------------------------------------------------------------
double double_gasdev_r(unsigned * pstate0, unsigned * pstate1);
double double_gasdev_ms_r(double mean, double stddev,
	unsigned * pstate0, unsigned * pstate1);
complex_double complex_double_gasdev_r(unsigned * pstate0, unsigned * pstate1);
complex_double complex_double_gasdev_ms_r(double mean, double stddev,
	unsigned * pstate0, unsigned * pstate1);

#endif // GASDEV_H
