#include "matrix_utils.h"

// ----------------------------------------------------------------
void pretty_print_double_matrix(
	tmatrix<double> & A,
	char * desc)
{
	int nr = A.get_num_rows();
	int nc = A.get_num_cols();
	printf("%s:\n", desc);
	for (int i = 0; i < nr; i++) {
		for (int j = 0; j < nc; j++) {
			if (j > 0)
				printf(" ");
			if (fabs(A[i][j]) > 1.0e6)
				printf(" %10.6le", A[i][j]);
			else
				printf(" %10.6lf", A[i][j]);
		}
		printf("\n");
	}
	printf("\n");
}

// ----------------------------------------------------------------
int compare_double_matrices(
	tmatrix<double> & actual,
	tmatrix<double> & expect,
	double            tol,
	double          & maxerr,
	double          & rmserr)
{
	int nr = actual.get_num_rows();
	int nc = actual.get_num_cols();
	double  maxmag = 0.0;

	for (int i = 0; i < nr; i++) {
		for (int j = 0; j < nc; j++) {
			double curmag = fabs(expect[i][j]);
			if (curmag > maxmag)
				maxmag = curmag;
		}
	}
	if (maxmag == 0.0)
		maxmag = 1.0;

	maxerr = 0.0;
	rmserr = 0.0;
	for (int i = 0; i < nr; i++) {
		for (int j = 0; j < nc; j++) {
			double diff = actual[i][j] - expect[i][j];

			double curerr = fabs(diff) / maxmag;
			if (curerr > maxerr)
				maxerr = curerr;
			rmserr += curerr * curerr;
		}
	}
	rmserr /= nr * nc;
	rmserr = sqrt(rmserr);

	if (rmserr <= tol)
		return 1;
	else
		return 0;
}
