#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "vector_print.h"

#define mypow(a,b) (exp((b)*log(a)))

// ================================================================
void complex_float_array_to_text_file(
	tvector<complex_float>  & array,
	char                    * file_name)
{
	complex_float_array_to_text_file(array.expose(), array.get_num_elements(), file_name);
}

// ----------------------------------------------------------------
void float_array_to_text_file(
	tvector<float>          & array,
	char                    * file_name)
{
	float_array_to_text_file(array.expose(), array.get_num_elements(), file_name);
}

// ----------------------------------------------------------------
void complex_double_array_to_text_file(
	tvector<complex_double> & array,
	char                    * file_name)
{
	complex_double_array_to_text_file(array.expose(), array.get_num_elements(), file_name);
}

// ----------------------------------------------------------------
void double_array_to_text_file(
	tvector<double>         & array,
	char                    * file_name)
{
	double_array_to_text_file(array.expose(), array.get_num_elements(), file_name);
}

// ================================================================
void complex_float_array_to_text_file(
	complex_float           * array,
	int                       length,
	char                    * file_name)
{
	FILE * fp = fopen(file_name, "w");
	if (fp == 0) {
		fprintf(stderr, "complex_float_array_to_text_file:  "
			"couldn't open \"%s\" for write.\n", file_name);
		exit(1);
	}
	for (int i = 0; i < length; i++) {
		fprintf(fp, "%11.7e, %11.7e # %08x %08x\n",
			array[i].re, array[i].im,
			*(unsigned *)&array[i].re,
			*(unsigned *)&array[i].im);
	}
	fclose(fp);
}

// ----------------------------------------------------------------
void float_array_to_text_file(
	float                   * array,
	int                       length,
	char                    * file_name)
{
	FILE * fp = fopen(file_name, "w");
	if (fp == 0) {
		fprintf(stderr, "float_array_to_text_file:  "
			"couldn't open \"%s\" for write.\n", file_name);
		exit(1);
	}
	for (int i = 0; i < length; i++) {
		fprintf(fp, "%11.7e # %08x\n",
			array[i], *(unsigned *)&array[i]);
	}
	fclose(fp);
}

// ----------------------------------------------------------------
void complex_double_array_to_text_file(
	complex_double          * array,
	int                       length,
	char                    * file_name)
{
	FILE * fp = fopen(file_name, "w");
	if (fp == 0) {
		fprintf(stderr, "complex_double_array_to_text_file:  "
			"couldn't open \"%s\" for write.\n", file_name);
		exit(1);
	}
	for (int i = 0; i < length; i++) {
		fprintf(fp, "%18.11lg, %18.11lg # %016llx %016llx\n",
			array[i].re, array[i].im,
			*(unsigned long long *)&array[i].re,
			*(unsigned long long *)&array[i].im);
	}
	fclose(fp);
}

// ----------------------------------------------------------------
void double_array_to_text_file(
	double                  * array,
	int                       length,
	char                    * file_name)
{
	FILE * fp = fopen(file_name, "w");
	if (fp == 0) {
		fprintf(stderr, "double_array_to_text_file:  "
			"couldn't open \"%s\" for write.\n", file_name);
		exit(1);
	}
	for (int i = 0; i < length; i++) {
		fprintf(fp, "%18.11lg # %016llx\n",
			array[i], *(unsigned long long *)&array[i]);
	}
	fclose(fp);
}
